@@ -10,7 +10,7 @@ use strict;
 use warnings;
 no warnings 'recursion'; # Disable the "deep recursion" warning
 
-our $VERSION = '0.48_002';
+our $VERSION = '0.48_003';
 our @ISA = qw( Future::_base );
 
 use Carp qw(); # don't import croak
@@ -821,7 +821,7 @@ sub wait_any
    # Look for immediate ready
    my $immediate_ready;
    foreach my $sub ( @subs ) {
-      $sub->{ready} and $immediate_ready = $sub, last;
+      $sub->{ready} and !$sub->{cancelled} and $immediate_ready = $sub, last;
    }
 
    if( $immediate_ready ) {
@@ -867,6 +867,7 @@ sub wait_any
 
    foreach my $sub ( @subs ) {
       # No need to test $sub->{ready} since we know none of them are
+      next if $sub->{cancelled};
       $sub->on_ready( $sub_on_ready );
       $pending++;
    }
@@ -888,17 +889,18 @@ sub needs_all
    my $self = Future->_new_convergent( \@subs );
 
    # Look for immediate fail
-   my $immediate_fail;
+   my $immediate_failure;
    foreach my $sub ( @subs ) {
-      $sub->{ready} and $sub->{failure} and $immediate_fail = $sub, last;
+      $sub->{cancelled} and $immediate_failure = [ "A component future was cancelled" ], last;
+      $sub->{ready} and $sub->{failure} and $immediate_failure = $sub->{failure}, last;
    }
 
-   if( $immediate_fail ) {
+   if( $immediate_failure ) {
       foreach my $sub ( @subs ) {
          $sub->{ready} or $sub->cancel;
       }
 
-      $self->{failure} = [ @{ $immediate_fail->{failure} } ];
+      $self->{failure} = [ @$immediate_failure ];
       $self->_mark_ready( "needs_all" );
       return $self;
    }
@@ -965,7 +967,7 @@ sub needs_any
    my $immediate_done;
    my $pending = 0;
    foreach my $sub ( @subs ) {
-      $sub->{ready} and !$sub->{failure} and $immediate_done = $sub, last;
+      $sub->{ready} and !$sub->{failure} and !$sub->{cancelled} and $immediate_done = $sub, last;
       $sub->{ready} or $pending++;
    }
 
